<?php
class App_Expert_WC_Product{
    private  $routesAllRegex="/\/wc\/store\/v1\/products(\/)?$/";
    private  $routesRegex="/\/wc\/store\/v1\/products\/[0-9]+$/";
    public function __construct() {
        //wc/v{x}/api
        add_filter('woocommerce_rest_prepare_product_object', array($this, 'add_products_edits'), 20, 3);
        add_filter('woocommerce_rest_product_object_query',array($this,'add_to_query'),10,2);
        add_filter( "rest_product_collection_params", array($this,'add_additional_params_to_product_api'), 10, 1 );
        //wc/store/v1/products
        add_filter('rest_request_after_callbacks', array($this, 'modify_product_data'), 999, 3);
        //api permission
        add_filter('woocommerce_rest_check_permissions',array($this, 'set_apis_permission'), 999, 4);
    }
    public function add_to_query($args, $request)
    {
        if (!empty($request['appexpert_attributes']) && is_array($request['appexpert_attributes'])) {

            $additional_tax_query = [];
            $all_available_attr_names = wc_get_attribute_taxonomy_names();
            foreach ($request['appexpert_attributes'] as $attribute_name => $attribute_term_ids) {
                if (in_array($attribute_name, $all_available_attr_names, true)) {
                    $additional_tax_query[] = array(
                        'taxonomy' => $attribute_name,
                        'field'    => 'term_id',
                        'terms'    => explode(",", $attribute_term_ids),
                    );
                }
            }
            if (!isset($args['tax_query'])) {
                $args['tax_query'] = [];
            }
            $args['tax_query'] = array_merge($additional_tax_query, $args['tax_query']); // WPCS: slow query ok.
        }

        if(isset($_GET['orderby'])){
            if ('rating' == $_GET['orderby']) {
                if(isset($_GET['order'])){
                    $args['orderby'] = 'meta_value_num';
                    $args['order'] = $_GET['order'];
                    $args['meta_key'] = '_wc_average_rating';
                }else{
                    $args['orderby'] = 'meta_value_num';
                    $args['order'] = 'DESC';
                    $args['meta_key'] = '_wc_average_rating';
                }
            }
        }

        return $args;
    }
    public function add_products_edits($response, $product)
    {
        if (empty($response->data))
            return $response;

        $response->data= App_Expert_WC_Product_Helper::get_data($response->data,$product);
        return $response;
    }
    public function add_additional_params_to_product_api ($params){
        $params['appexpert_attributes'] = array(
            'description' => __( '(Array) filter with multiple attribute. example:appexpert_attributes[pa_color]=20,30' ),
            'type'        => 'Array',
        );
        return $params;
    }


    public function modify_product_data($response, array $handler, \WP_REST_Request $request) {
        if( $response instanceof WP_Error) return $response;
        $current_route = $request->get_route();
        $matches=[];
        //list of products
        preg_match($this->routesAllRegex,$current_route,$matches);
        if(count($matches)){
            return $this->edit_list($response);
        }
        preg_match($this->routesRegex,$current_route,$matches);
        if(count($matches)){
            return $this->edit_one($response);
        }
        return $response;
    }
    private function edit_list(WP_REST_Response $response)
    {
        $new_data=[];
        foreach ($response->get_data() as $product){
            $new_data[]= App_Expert_WC_Product_Helper::get_data($product);
        }
        $response->set_data($new_data);
        return $response;
    }
    private function edit_one(WP_REST_Response $response)
    {
        $response->set_data(App_Expert_WC_Product_Helper::get_data($response->get_data()));
        return $response;
    }

    public function set_apis_permission($permission, $context, $object_id, $post_type){
        if($post_type =="product"&&$context=="read") return true;
        return $permission;
    }
}
new App_Expert_WC_Product();